#include "read_energ_data.i"

func attach_std_arf( specfile, offaxis, det=, aps=, effabundle= )
/* DOCUMENT attach_std_arf, specfile, offaxis, det=, aps=, effabundle=

     Assume that the spectral file 'specfile' is in PHAII format

     It generates an ARF from the supplied 'offaxis' value (in arcmin).

     The default 'effabundle' i.e. telescope effective area file is:
           ~/0idl/imd/pf16_sat00_105_aps3/Eff_area_FM[12]_v12_aps3_130308.fits

     Keyword:    det         the detector number (defaults to 0). Used for the naming
                             of the RMF file.
                 aps         the aperture stop correction table (filename) to apply
                 effabundle  the assembled effective area file

   2013-01-28/NJW, cloned from 'prep_spec.i'
*/
{

  /*
   *    Construct the ARF from the optic effective area and the
   *    thermal foil absorption
   */

   offaxis = double(offaxis);
   hdr = headfits(specfile+"+1");
   instrume = fxpar(hdr,"instrume");
   mstr = instrume == "FPMA" ? "FM2" : "FM1";

   if( is_void(effabundle) ) \
      effabundle = "/home/njw/0idl/imd/pf16_sat00_105_aps3/Eff_area_"+mstr+"_v12_aps3_130308.fits";
   if( !file_test(effabundle) ) error,"File not found: "+effabundle;

   effa_dol = effabundle+"+1";
   offaxis_arcmin = (rdfitscol(effa_dol, "offaxis")(2:0))/60.;
   arr = rdfitscol(effa_dol, "eff_area" );
   ener = arr(,1);
   effa = arr(,2:0);

   arfname = get_next_filename(mstr+"_std_????.arf");

   // interpolate
   w = where( offaxis_arcmin <= offaxis );
   i = w(0);
   if( i >= numberof(offaxis_arcmin) ) error,"Requested offaxis is too large";
   f = (offaxis - offaxis_arcmin(i))/(offaxis_arcmin(i+1)-offaxis_arcmin(i));
   eff_area = (1-f)*effa(,i) + f*effa(,i+1);
   tr = read_energ_data("transmis_bemli.fits","transmis");
   arf = tr.data * interp(eff_area,ener,tr.el);

   kwds_init;
   kwds_set,"OFFAXIS", offaxis, "[arcmin] Off-axis angle";
   kwds_setlongstr,"EFFAFILE", effabundle, "Effective area input file";

   if( !is_void(aps) ) {
      if( !file_test(aps) ) error,"Did not find "+aps;
      apscor = read_energ_data( aps, "specresp" );
      arf *= interp(apscor.data, apscor.el, tr.el );
      kwds_setlongstr, "APSCOR", aps, "Aperture stop correction file";
   }

   // Create the ARF file

   arf2phaii, arfname, arf, tr.elo, tr.ehi, extname="SPECRESP", \
      instrume=instrume, telescop="NuSTAR",nokwdsinit=1;
   write,"Wrote "+arfname;

   // Update the ANCRFILE column in the spectral file

   attach_arf, specfile, arfname;
   write,specfile+" has had its ANCRFILE updated to: "+arfname;

}
