struct s_Parval {
   int model;
   string component;
   string name;
   string unit;
   double value;
   string status;
   double value_err;
}
struct s_Fitqual {
   int dof;
   double red_chi2;
}

func get_xspec_fit_params( xspec_log_file, &fitqual, flux=, outfile=, silent= )
/* DOCUMENT get_xspec_fit_params, xspec_log_file, >fitqual, flux=, outfile=, silent=
    or      res = get_xspec_fit_params( xspec_log_file, >fitqual, flux=, outfile=, silent= )

   Assumes that the fitting parameters are found between a line with
     =================
   and
     _________________

  Extract power law fitting parameters from XSPEC logfile

  Returns an array of structs 's_Parval' with elements:
     model, component, name, unit, value, value_err, status

  and a struct 'fitqual' with elements red_chi2 and dof.

  If the keyword 'flux' is set then it returns a 5x2-element array where
  the flux is the last number.

 2004-06-03/NJW, original IDL version
 2008-08-12/NJW translated to Yorick
 2011-03-31/NJW updated with keyword 'flux'
 2013-01-18/NJW More general version
*/
{


   logfile = xspec_log_file;

   flag = typeof( outfile ) == "string";
   if( flag ) lun = open( outfile,"a");
   
   
   lines = rdfile(logfile);
   nlines = numberof(lines);
   
   // Search for the presence of "======================="

   idx1 = where(strmatch(lines,"================================"));
   idx2 = where(strmatch(lines,"________________________________"));
   nidx1 = numberof(idx1);
   if( nidx1 != numberof(idx2) ) error,"##1##";

   nparams = idx2(0) - idx1(0) - 4;
   idx1 = idx1(0) + 4;
   idx2 = idx2(0) - 1;
   if(!silent)prstrarr, lines(idx1:idx2+10);

   // Locate 'Reduced chi-squared' following the parameter list

   fitqual = s_Fitqual();
   idx3 = where(strmatch(lines(idx2+2:0),"Reduced chi-squared"));
   if( numberof(idx3) > 0 ) {
      line = lines(idx3(1)+idx2+1);
      p = strpos(line,"squared")+9;
      pf = strpos(line,"for",p);
      pd = strpos(line,"degrees",pf);
      fitqual.red_chi2 = atof(strpart(line,p:pf-1));
      fitqual.dof = atoi(strpart(line,pf+3:pd-1));
   } else {
      fitqual.red_chi2 = 0.0;
      fitqual.dof = 0;
   }

   res = array( s_Parval, nparams );

   for( i = 1; i <= nparams; i++ ) {
      line = lines(idx1 + i - 1);
      res(i).model = atoi(strpart(line, 6:10));
      res(i).component = strtrim(strpart(line,11:24));
      res(i).name = strtrim(strpart(line,25:35));
      res(i).unit = strtrim(strpart(line,36:43));
      res(i).value = atof(strpart(line,44:55));
      rem = strtrim(strpart(line,56:0));
      if( strpos(rem,"+/-") ) { // then an error is present
         res(i).value_err = atof(strpart(rem,4:0));
         res(i).status = "active";
      } else if( strpos(rem,"frozen") ) { // parameter is frozen
         res(i).value_err = 0.0;
         res(i).status = "frozen";
      } else { // unknown status
         res(i).value_err = 0.0;
         res(i).status = "indetermined";
      }
      if( am_subroutine() ) {
         if( res(i).status == "active" ) {
            if(!silent)write,format="%6i%6i%10s%10s%10s%14.6f%14.6f\n", i, res(i).model, \
               res(i).component, res(i).name, res(i).unit, res(i).value, res(i).value_err;
            if( flag ) {
               write,lun,format="%6i%6i%10s%10s%10s%14.6f%14.6f\n", i, res(i).model, \
                  res(i).component, res(i).name, res(i).unit, res(i).value, res(i).value_err;
            }
         } else {
            if(!silent)write,format="%6i%6i%10s%10s%10s%14.6f%10s\n", i, res(i).model, \
               res(i).component, res(i).name, res(i).unit, res(i).value, res(i).status;
            if( flag ) {
               write,lun,format="%6i%6i%10s%10s%10s%14.6f%10s\n", i, res(i).model, \
                  res(i).component, res(i).name, res(i).unit, res(i).value, res(i).status;
            }
         }
      }
   }
   if( flag ) close, lun;
   return res;
}

gx = get_xspec_fit_params;
write,"Now you have shorthand: gx  for get_xspec_fit_params";
